/*
 * file Version.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 *
 * (c) Copyright IBM Corporation 2004, 2008.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 */
package javax.wvcm;

import java.util.List;

import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.WvcmException.ReasonCode;

/**
 * A proxy for a version resource.
 * 
 * A version resource contains an immutable copy of the properties of a
 * version-controlled resource. A version resource also contains mutable
 * properties with metadata about that version.
 * 
 * @since 1.0
 */
public interface Version extends Resource {

    /**
     * Get the workspace provider of this resource.
     * 
     * @return the {@link WorkspaceProvider} for this Resource.
     */
    public WorkspaceProvider workspaceProvider();

    /**
     * The {@link Activity} object that identify the
     * activity to which this version contributes.
     * A provider may refuse to allow the activity of a version to be modified.
     * This version must be on the same line of descent
     * as all other versions with this activity from the same version history.
     * If the {@link Activity#ACTIVITY_CHECKOUT_LIST} of the activity identifies a
     * controllable resource that is reserved and checks out a version
     * in the version history of this version,
     * then the checked-out version must be a successor of all other
     * versions with this activity from the same version history. 
     * @see #setActivity
     * @see #getActivity
     */
    public static final PropertyName<Activity> ACTIVITY =
        new PropertyName<Activity>("activity"); //$NON-NLS-1$

    /**
     * Get the {@link #ACTIVITY} property.
     * 
     * @return the {@link #ACTIVITY} property.
     * @throws WvcmException if this property was not set and
     * this Version was not created with
     * {@link #ACTIVITY} as a wanted property.
     * @see #setActivity
     */
    public Activity getActivity() throws WvcmException;

    /**
     * Set the {@link #ACTIVITY} property.
     * 
     * @param activity an {@link Activity} object that specifies
     * the new {@link #ACTIVITY} property for this Version.
     * @see #getActivity
     */
    public void setActivity(Activity activity);

    /**
     * Valid values for the {@link #CHECKIN_FORK} and {@link #CHECKOUT_FORK} properties.
     */
    public enum Fork {

        /**
         * Forking is allowed without special options.
         */
        OK("ok"), //$NON-NLS-1$

        /**
         * Forking is allowed only if specifically requested.
         */
        DISCOURAGED("discouraged"), //$NON-NLS-1$

        /**
         * Forking is not allowed.
         */
        FORBIDDEN("forbidden"); //$NON-NLS-1$

        private final String _forkType;

        private Fork(String forkType) {
            _forkType = forkType;
        }

        /**
         * Returns a string representation of this Fork suitable for diagnostics.
         */
        @Override
        public String toString() { return _forkType; }
    }

    /**
     * An enumeration that determines whether this Version can be given
     * more than one successor.
     * If {@link #CHECKIN_FORK} is {@link Fork#DISCOURAGED}, a doCheckin request
     * MUST fail unless forkOk is specified in the doCheckin request.
     * If the {@link #CHECKIN_FORK} of a version is {@link Fork#FORBIDDEN},
     * a doCheckin request MUST fail if it would result in that version
     * appearing in the {@link #PREDECESSOR_LIST} of more than one version.
     * A server MAY reject attempts to modify the {@link #CHECKIN_FORK} of a version.
     * @see #setCheckinFork
     * @see #getCheckinFork
     */
    public static final PropertyName<Fork> CHECKIN_FORK =
        new PropertyName<Fork>("checkin-fork"); //$NON-NLS-1$

    /**
     * Get the {@link #CHECKIN_FORK} property.
     * 
     * @return the {@link #CHECKIN_FORK} property.
     * @throws WvcmException if this property was not set and
     * this Version was not created with
     * {@link #CHECKIN_FORK} as a wanted property.
     * @see #setCheckinFork
     */
    public Fork getCheckinFork() throws WvcmException;

    /**
     * Set the {@link #CHECKIN_FORK} property.
     * 
     * @param val the new {@link #CHECKIN_FORK} value for this Version.
     * @see #getCheckinFork
     */
    public void setCheckinFork(Fork val);

    /**
     * An enumeration that determines whether a version-controlled resource
     * selecting this Version can be checked out when it already has
     * a successor version.
     * If {@link #CHECKOUT_FORK} is {@link Fork#DISCOURAGED}, a doCheckout request MUST fail
     * unless forkOk is specified in the doCheckout request.
     * If the {@link #CHECKOUT_FORK} of a version is {@link Fork#FORBIDDEN},
     * a doCheckout request will fail if it would result in that version
     * appearing in the {@link #PREDECESSOR_LIST} or {@link ControllableResource#CHECKED_OUT} property
     * of more than one version or checked-out resource.
     * A server MAY reject attempts to modify the {@link #CHECKOUT_FORK} of a version.
     * @see #setCheckoutFork
     * @see #getCheckoutFork
     */
    public static final PropertyName<Fork> CHECKOUT_FORK =
        new PropertyName<Fork>("checkout-fork"); //$NON-NLS-1$

    /**
     * Get the {@link #CHECKOUT_FORK} property.
     * 
     * @return the {@link #CHECKOUT_FORK} property.
     * @throws WvcmException if this property was not set and
     * this Version was not created with
     * {@link #CHECKOUT_FORK} as a wanted property.
     * @see #setCheckoutFork
     */
    public Fork getCheckoutFork() throws WvcmException;

    /**
     * Set the {@link #CHECKOUT_FORK} property.
     * 
     * @param val the new {@link #CHECKOUT_FORK} value for this Version.
     * @see #getCheckoutFork
     */
    public void setCheckoutFork(Fork val);

    /**
     * A list of all controllable resources that are checked out from this version.
     * This is the computed inverse of the {@link ControllableResource#CHECKED_OUT} property.
     * @see #getCheckoutList
     */
    public static final PropertyName<ResourceList<ControllableResource>> CHECKOUT_LIST =
        new PropertyName<ResourceList<ControllableResource>>("checkout-list"); //$NON-NLS-1$

    /**
     * Get the {@link #CHECKOUT_LIST} property.
     * 
     * @return the {@link #CHECKOUT_LIST} property.
     * @throws WvcmException if this Version was not created with
     * {@link #CHECKOUT_LIST} as a wanted property.
     */
    public ResourceList<ControllableResource> getCheckoutList()
    throws WvcmException;

    /**
     * A list of strings that identify the labels
     * that currently select this Version.
     * @see #getLabelNameList
     */
    public static final PropertyName<List<String>> LABEL_NAME_LIST =
        new PropertyName<List<String>>("label-name-list"); //$NON-NLS-1$

    /**
     * Get the {@link #LABEL_NAME_LIST} property.
     * @return the {@link #LABEL_NAME_LIST} property.
     * @throws WvcmException if this Version was not created with
     * {@link #LABEL_NAME_LIST} as a wanted property.
     */
    public List<String> getLabelNameList() throws WvcmException;

    /**
     *  A list of all versions that are direct predecessors of this version.
     * @see #getPredecessorList
     */
    public static final PropertyName<ResourceList<Version>> PREDECESSOR_LIST =
        new PropertyName<ResourceList<Version>>("predecessor-list"); //$NON-NLS-1$

    /**
     * Get the {@link #PREDECESSOR_LIST} property.
     * @return the {@link #PREDECESSOR_LIST} property.
     * @throws WvcmException if this Version was not created with
     * {@link #PREDECESSOR_LIST} as a wanted property.
     */
    public ResourceList<Version> getPredecessorList() throws WvcmException;

    /**
     * A list of all versions that are direct successors of this version.
     * The {@link #SUCCESSOR_LIST} property is the computed inverse of the
     * {@link #PREDECESSOR_LIST} property.
     * @see #getSuccessorList
     */
    public static final PropertyName<ResourceList<Version>> SUCCESSOR_LIST =
        new PropertyName<ResourceList<Version>>("successor-list"); //$NON-NLS-1$

    /**
     * Get the {@link #SUCCESSOR_LIST} property.
     * 
     * @return the {@link #SUCCESSOR_LIST} property.
     * @throws WvcmException if this Version was not created with
     * {@link #SUCCESSOR_LIST} as a wanted property.
     */
    public ResourceList<Version> getSuccessorList() throws WvcmException;

    /**
     * The version history that contains this version.
     * @see #getVersionHistory
     */
    public static final PropertyName<VersionHistory> VERSION_HISTORY =
        new PropertyName<VersionHistory>("version-history"); //$NON-NLS-1$

    /**
     * Get the {@link #VERSION_HISTORY} property.
     * 
     * @return the {@link #VERSION_HISTORY} property.
     * @throws WvcmException if this Version was not created with
     * {@link #VERSION_HISTORY} as a wanted property.
     */
    public VersionHistory getVersionHistory() throws WvcmException;

    /**
     * A server-defined string that is different for each version
     * in the version history of this version.
     * This string is intended for display to a user, 
     * unlike the location of a version, which is normally only used by a client
     * and not displayed to a user.
     * @see #getVersionName
     */
    public static final PropertyName<String> VERSION_NAME =
        new PropertyName<String>("version-name"); //$NON-NLS-1$

    /**
     * Get the {@link #VERSION_NAME} property.
     * 
     * @return the {@link #VERSION_NAME} property.
     * @throws WvcmException if this Version was not created with
     * {@link #VERSION_NAME} as a wanted property.
     */
    public String getVersionName() throws WvcmException;

    /**
     * Add the specified label to this version.
     * 
     * @param label the label to be added to this version
     * @param feedback Specifies optional feedback to the caller.
     * @return A new proxy for this resource, whose properties are specified by feedback.
     * @throws WvcmException
     * <li>{@link ReasonCode#ADD_MUST_BE_NEW_LABEL}:
     *  The label MUST NOT already identify a version in the version history of this version.
     */
    public Version doAddLabel(String label, Feedback feedback) throws WvcmException;

    /**
     * Remove the specified label from this version.
     * 
     * @param label the label to be removed from this version.
     * @param feedback Specifies optional feedback to the caller.
     * @return A new proxy for this resource, whose properties are specified by feedback.
     * @throws WvcmException ReasonCode:
     * <li>{@link ReasonCode#CANNOT_REMOVE_LABEL_DOES_NOT_EXIST}:
     *  The label must be on this version.
     */
    public Version doRemoveLabel(String label, Feedback feedback) throws WvcmException;

    /**
     * Set the specified label on this version.
     * 
     * @param label The label to be set on this version
     * @param feedback Specifies optional feedback to the caller.
     * @return A new proxy for this resource, whose properties are specified by feedback.
     * @throws WvcmException if label cannot be set
     */
    public Version doSetLabel(String label, Feedback feedback) throws WvcmException;

    /**
     * A list of all workspaces that have a controllable resource that contains
     * this version in its {@link ControllableResource#CHECKED_IN} or
     * {@link ControllableResource#CHECKED_OUT} property.
     * Because there can be many workspaces that contain a given version,
     * this property is most commonly used in a {@link Resource#doFind} request.
     * @see #getInWorkspaceList
     */
    public static final PropertyName<ResourceList<Workspace>> IN_WORKSPACE_LIST =
        new PropertyName<ResourceList<Workspace>>("in-workspace-list"); //$NON-NLS-1$

    /**
     * Get the {@link #IN_WORKSPACE_LIST} property.
     * 
     * @return the {@link #IN_WORKSPACE_LIST} property.
     * @throws WvcmException if this Version was not created with
     * {@link #IN_WORKSPACE_LIST} as a wanted property.
     */
    public ResourceList<Workspace> getInWorkspaceList()
    throws WvcmException;

    /**
     * A list of all baselines that contain this version in their {@link Baseline#VERSION_LIST}
     * property.
     * This is the computed inverse of the {@link Baseline#VERSION_LIST} property.
     * Because there can be many baselines that contain a given version,
     * this property is most commonly used in a {@link Resource#doFind} request.
     * @see #getInBaselineList
     */
    public static final PropertyName<ResourceList<Baseline>> IN_BASELINE_LIST =
        new PropertyName<ResourceList<Baseline>>("in-baseline-list"); //$NON-NLS-1$

    /**
     * Get the {@link #IN_BASELINE_LIST} property.
     * 
     * @return the {@link #IN_BASELINE_LIST} property.
     * @throws WvcmException if this Version was not created with
     * {@link #IN_BASELINE_LIST} as a wanted property.
     */
    public ResourceList<Baseline> getInBaselineList()
    throws WvcmException;

}
